import invariant from 'invariant';
import _property from 'lodash/property';
import _reduce from 'lodash/reduce';
import _isEmpty from 'lodash/isEmpty';
import _includes from 'lodash/includes';
import { getReduxMountPoint, DEFAULT_REDUX_MOUNT_POINT } from './mountPoint';
export function getInputsFromState() {
  var inputConfigOrMountPoint = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_REDUX_MOUNT_POINT;
  var state = arguments.length > 1 ? arguments[1] : undefined;
  var inputKeys = arguments.length > 2 ? arguments[2] : undefined;
  var mountPoint = typeof inputConfigOrMountPoint === 'string' ? inputConfigOrMountPoint : getReduxMountPoint(inputConfigOrMountPoint);

  var inputsState = _property(mountPoint)(state);

  invariant(inputsState, "[redux-inputs]: no state found at '".concat(mountPoint, "', check your reducers to make sure it exists or change reduxMountPoint in your inputConfig."));

  if (inputKeys) {
    return _reduce(inputsState, function (result, input, key) {
      if (_includes(inputKeys, key)) {
        result[key] = input;
      }

      return result;
    }, {});
  } else {
    return inputsState;
  }
}
export function inputsWithErrors(inputs) {
  var erroredInputs = _reduce(inputs, function (result, input, key) {
    if (typeof input.error !== 'undefined') {
      result[key] = input;
    }

    return result;
  }, {});

  return _isEmpty(erroredInputs) ? false : erroredInputs;
}